/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#define F_CPU 8000000UL
#include <util/delay.h>
#include <avr/interrupt.h>
#include "lcd.h"

/******************************************************************************
* Macros
******************************************************************************/


/******************************************************************************
* Constants
******************************************************************************/


/******************************************************************************
* Global Variables
******************************************************************************/


/******************************************************************************
* External Variables
******************************************************************************/


/******************************************************************************
* Local Function Declarations
******************************************************************************/
void lcdre(char data, uint8_t rs);
void pulz(void);
void lcd_function_set(uint8_t bit, uint8_t line, uint8_t font);

/******************************************************************************
* Local Function Definitions
******************************************************************************/
/******************************************************************************
* Function:		void lcdre(uint8_t data, uint8_t rs, uint8_t rw, uint8_t csakalso)
* Description:	LCD kezel fggvny
* Input:		adatbitek, rs
* Output:
* Notes:		alacsony szint fggvny
******************************************************************************/
void lcdre(char data, uint8_t rs)
{
	_delay_us(500);
	if (rs<2)
	{
		if (rs==TRUE)
		{
			rs=0b00000100;
		}
		
		uint8_t also;
		uint8_t felso;
		
		also = data << 4;
		felso = data & 0xF0;
		
		PORTC=felso | rs;
		pulz();
		PORTC= also | rs;
		pulz();
		
	}
}

/******************************************************************************
* Function:		void pulz(void)
* Description:	Pulzl jel adsa az LCD-nek
* Input:
* Output:
* Notes:		minden parancs utn a mvelet vgrehajtshoz
******************************************************************************/
void pulz(void)
{
	_delay_ms(2);
	PORTC=PORTC ^ 0b00001000;
	_delay_us(5);
	PORTC=PORTC ^ 0b00001000;

}

/******************************************************************************
* Function:         void lcd_function_set(uint8_t bit, uint8_t line, uint8_t font)
* Description:		mkds belltsa(inicializlshoz)
* Input:			bit,line,font
* Output:
* Notes:
******************************************************************************/
void lcd_function_set(uint8_t bit, uint8_t line, uint8_t font)
{
	uint8_t data=0;
	if (bit<2 && line<2 && font<2)
	{
		
		if (bit==D4_BIT)
		{
			data|=0b00000000;
		}
		
		if (bit==D8_BIT)
		{
			data|=0b00010000;
		}
		
		if (line==TWO_LINE)
		{
			data|=0b00001000;
		}
		
		if (font==FONT5X10)
		{
			data|=0b00000100;
		}
		
		data|=0b00100000;
		
		lcdre(data,0);
	}
}

/******************************************************************************
* Global Function Definitions
******************************************************************************/

/******************************************************************************
* Function:         void set_address(uint8_t address)
* Description:		Az LCD memriacmre val ugrs
* Input:			cm (0-16 els sor; 40-67 msodik sor)
* Output:
* Notes:
******************************************************************************/
void lcd_set_address(uint8_t address)
{
	if (address < 67)
	{
		lcdre(0b10000000 | address,0);
	}
	
}

/******************************************************************************
* Function:         void lcd_clrdisplay(void)
* Description:		Kijelz trlse
* Input:
* Output:
* Notes:
******************************************************************************/
void lcd_clrdisplay(void)
{
	lcdre(0b00000001,0);
}

/******************************************************************************
* Function:         void lcd_rethome(void)
* Description:		A kurzor kijelz elejre mozgatsa
* Input:
* Output:
* Notes:
******************************************************************************/
void lcd_rethome(void)
{
	lcdre(0b00000010,0);
}

/******************************************************************************
* Function:         void lcd_modeset(uint8_t incordec, uint8_t shiftornotshift)
* Description:		Md belltsa(inicializlshoz)
* Input:			incordec, shiftornotshift
* Output:
* Notes:
******************************************************************************/
void lcd_modeset(uint8_t incordec, uint8_t shiftornotshift)
{
	uint8_t data=0;
	if (incordec<2 && shiftornotshift<2)
	{
		if (incordec==INCREMENT)
		{
			data|=0b00000010;
		}
		
		if (shiftornotshift==SHIFT)
		{
			data|=0b00000001;
		}
		
		data|=0b00000100;
		
		lcdre(data,0);
	}
}

/******************************************************************************
* Function:         void lcd_displayonof(uint8_t disponof, uint8_t cursoronof, uint8_t blinksonof)
* Description:		Kijelz bekapcsolsa(inicializlshoz)
* Input:			disponof, cursoronof, blinksonof
* Output:
* Notes:
******************************************************************************/
void lcd_displayonof(uint8_t disponof, uint8_t cursoronof, uint8_t blinksonof)
{
	uint8_t data=0;
	if (disponof<2 && cursoronof<2 && blinksonof<2)
	{
		if (disponof==ON)
		{
			data|=0b00000100;
		}
		
		if (cursoronof==ON)
		{
			data|=0b00000010;
		}
		
		if (blinksonof==ON)
		{
			data|=0b00000001;
		}
		
		data|=0b00001000;
		
		lcdre(data,0);
	}
}

/******************************************************************************
* Function:         void lcd_shift(uint8_t incdec, uint8_t displshift, uint8_t hanyszor)
* Description:		Lptets
* Input:			incdec, displshift, hanyszor
* Output:
* Notes:
******************************************************************************/
void lcd_shift(uint8_t incdec, uint8_t displshift, uint8_t hanyszor)
{
	uint8_t data=0,e=0;
	if (incdec<2 && displshift<2)
	{
		if (incdec==INCREMENT)
		{
			data|=0b00000100;
		}
		
		if (displshift==DISP_SHIFT)
		{
			data|=0b00001000;
		}
		
		data|=0b00010000;
		
		for (e=0;e<hanyszor;e++)
		{
			lcdre(data,0);
		}
		
	}
}

/******************************************************************************
* Function:		uint8_t lcdreir(char stri[])
* Description:	Karaktersorozat rsa az LCD kijelzre
* Input:		karaktersorozat
* Output:		a sorozat hossza
* Notes:
******************************************************************************/
uint8_t lcdreir(char stri[])
{
	int i=0;
	while(stri[i]!='\0')
	{
		lcdre(stri[i],1);
		i++;
	}
	return i;
}



/******************************************************************************
* Function:		void lcdinit(void)
* Description:	LCD inicializlsa
* Input:
* Output:
* Notes:		inicializl algoritmus(tapasztalati ton kerlt meghatrozsra)
******************************************************************************/
void lcdinit(void)
{
	//PORT DDR SET
	DDRC = LCD_mask;  
	_delay_ms(10);
	//4 bites md belltsa (Az LCD jelenleg 8 bites mdban van.)
	PORTC=0b00110000;
	pulz();
	PORTC=0b00110000;
	pulz();
	PORTC=0b00110000;
	pulz();
	PORTC=0b00100000;
	pulz();
	//LCD inicializlsi procedra
	lcd_function_set(D4_BIT,TWO_LINE,FONT5X10);
 	lcd_displayonof(ON,OFF,OFF);	
	lcd_clrdisplay();
	lcd_modeset(INCREMENT,NOT_SHIFT);
	_delay_ms(100);
}